/*
 *	Folder manager implementation
 *	issued: 2000/4/3
 *	author: c.mos@zob.ne.jp
 */

#include "ged.h"
#include "gui.h"
#include "folder.h"
#include "icons.h"
#include "prof.h"
#include "remote.h"

#define FOLDER_X 0
#define FOLDER_Y 2
#define ICON_COUNT_X 4
#define ICON_COUNT_Y 2
#define LIST_COUNT (STATUS_Y - FOLDER_Y - 1)
#define REGION_CX 7
#define REGION_CY 7
#define ICON_SIZE 4
#define NAME_WIDTH 6
#define NAME_HIGHT 2

#define MAX_FILES 32
#define MAX_ICONREF 32

#define LIST_NAME_LEN 9
#define LIST_INFO_LEN 12

#define APP_MARK 0x02
#define IL_MARK 0x0F
#define UP_MARK 0x1E
#define DOWN_MARK 0x1F

extern char *fs_name[FS_COUNT];

IconRef icon_ref[MAX_ICONREF];
int icon_ref_count = 0;

static int icontable[MAX_FILES];
WORD get_fileicon(fent_t __ss* fs);

/* #define ICON_COUNT 6 */

void init_folder(Folder* folder, int redraw)
{
	int index;
	set_current_fs(folder->fsid);
	set_mask(folder->flags & FOLDER_SYS ? 0 : FMODE_ILIB);
	folder->files = get_file_count();
	if(redraw != TRUE) {
		if(redraw == SELECT_NAME) {
			folder->select = select_file(folder->fname);
		} else { /* SELECT_CUR */
			if(folder->select >= folder->files)
				folder->select = folder->files - 1;
			if(folder->select < 0) goto Select1st;
		}
		if(folder->flags & FOLDER_LIST)
			folder->home = folder->select < LIST_COUNT ? 0 : folder->select - LIST_COUNT + 1;
		else
			folder->home = (folder->select < (ICON_COUNT_X * ICON_COUNT_Y)) ? 0
							: (folder->select / ICON_COUNT_X - 1) * ICON_COUNT_X;
	} else
Select1st:
		folder->home = folder->select = 0;

	for_to(index, folder->files) {
		fent_t fent;
		int i;
		char far *ext;
		if(get_entry_tmp(index, &fent)) {
			WORD iconp = NULL;
			if(folder->fsid == FS_FLASH) iconp = get_fileicon(&fent);
			if(iconp)
				icontable[index] = (int)iconp;
			else {
				int icon;
				i = find_filetype(folder->fsid, &fent, icon_ref[0].name, sizeof(IconRef), icon_ref_count);
				if(i >= 0)
					icon = icon_ref[i].icon;
				else {
					WORD mode = fent.mode;
					if(mode & FMODE_ILIB) icon = ICON_ILIB;
					else if(mode & FMODE_X) icon = ICON_APP;
					else icon = ICON_TEXT;
				}
				icontable[index] = icon;
			}
		}
	}
}

WORD get_fileicon(fent_t __ss *fs)
{
	res_image_t far *res;
	if(fs->resource == -1L) return NULL;
	res = (res_image_t far *)((BYTE far*)get_location(fs->loc) + fs->resource);
	while(res->header.magic == RESID) {
		if(res->header.type == RESID_IMAGE && res->width == 4) {
			return far2seg(res + 1);
		}
		(BYTE far*)res += (res->header.size << 4);
	}
	return NULL;
}

int find_filetype(FSID fsid, FS fs, char *arr_name, size_t arraysize, int count)
{
	int i;
	res_t res;
	char far *ext = strrchr(fs->name, '.');
	res.type = 0;
	if(fs->resource == 0L) {
		read_resource_header(fsid, fs, &res);
	}
	for_to(i, count) {
		if(*arr_name == '.') {
			if(ext && !strncmp(arr_name, ext, MAXFNAME))
				return i;
		} else if(*arr_name == '&') {
			if(res.type == (arr_name[1] |  (arr_name[2] << 8) ))
				return i;
		} else {
			if(!strncmp(arr_name, fs->name, MAXFNAME))
				return i;
		}
		arr_name += arraysize;
	}
	return -1;
}

void draw_filename(Rect __ss *r, char far *name)
{
	int y1 = 0;
	char far *p = name;
	char far *p1;
	for_to(y1, r->cy) {
		int x, y, cx;
		int len = string_skip(p, r->cx);
		int n = len;
		while(n >= 0 && (BYTE)*(p+n) > ' ') n--;
		if(n > 0) len = n;
		cx = string_width(p, len);
		if(cx >= r->cx) cx = r->cx;
		x = r->x + (r->cx - cx) / 2;
		y = r->y + y1;
		text_put_far_substring(x, y, p, cx);
		p += len;
		while(*p && (BYTE)*p <= ' ') p++;
		if(!*p) break;
	}
}

void draw_scroll_arrow(Folder* folder)
{
	int count = (folder->flags & FOLDER_LIST) ? LIST_COUNT : ICON_COUNT_X * ICON_COUNT_Y;
	char up_arrow = folder->home ? UP_MARK : ' ';
	char down_arrow = (folder->home + count < folder->files) ? DOWN_MARK : ' ';
	set_color(CLR_DESKTOP);
	text_put_char(LCD_CHAR_CX-1, FOLDER_Y, up_arrow);
	text_put_char(LCD_CHAR_CX-1, FOLDER_Y + 1, up_arrow);
	text_put_char(LCD_CHAR_CX-1, FOLDER_Y + REGION_CY*2 -3, down_arrow);
	text_put_char(LCD_CHAR_CX-1, FOLDER_Y + REGION_CY*2 -2, down_arrow);
}

void draw_icon(int x, int y, int index)
{
	int icon = icontable[index];
	put_color_icon(x, y, ICON_SIZE, ICON_SIZE, icon < 0 ? MK_FP((WORD)icon, 0) : (WORD far*)get_bmpres(BMP_ICONS) + icon * (8*16));
}

void draw_icon_on_desk(Folder* folder, int index)
{
	fent_t fent;
	Rect r;
	int i;
	i = index - folder->home;
	r.x = (i % ICON_COUNT_X) * REGION_CX + FOLDER_X;
	r.y = (i / ICON_COUNT_X) * REGION_CY + FOLDER_Y;
	r.cx = NAME_WIDTH; r.cy = NAME_HIGHT;
	draw_icon(r.x + 1, r.y, index);
	set_color(index == folder->select ? CLR_HIGHLIGHT : CLR_DESKTOP);
	r.y += 4;
	if(get_entry_tmp(index, &fent))
		draw_filename(&r, fent.info);
}

void draw_icon_folder(Folder* folder, Rect *r_clip)
{
	int i;
	static Rect r, r1;
	r.cx = REGION_CX; r.cy = REGION_CY;
	for_to(i, ICON_COUNT_X * ICON_COUNT_Y) {
		r.x = (i % ICON_COUNT_X) * REGION_CX + FOLDER_X;
		r.y = (i / ICON_COUNT_X) * REGION_CY + FOLDER_Y;
		if(!r_clip || intersect_rect(&r1, &r, r_clip)) {
			fill_rect(r_clip ? &r1 : &r, PAL_DESKTOP, CLR_DESKTOP);
			if(i + folder->home < folder->files)
				draw_icon_on_desk(folder, i + folder->home);
		}
	}
}

/* filename x 123K informations */

void draw_list1(Folder* folder, int index)
{
	fent_t fent;
	int x = 0;
	int y = index - folder->home + FOLDER_Y;
	set_color(index == folder->select ? CLR_HIGHLIGHT : CLR_DESKTOP);
	if(get_entry_tmp(index, &fent)) {
		char c = (fent.mode & FMODE_ILIB) ? IL_MARK : (fent.mode & FMODE_X) ? APP_MARK : ' ';
		WORD len_kb = (WORD)_lrotr(fent.len, 10);
		if(len_kb == 0) len_kb = 1;
		*((char _ss*)fent.name + string_skip(&fent.name, LIST_NAME_LEN)) = '\0';
		*((char _ss*)fent.info + string_skip(&fent.info, LIST_INFO_LEN)) = '\0';
		x = text_printf(0, y, "%c%-9ls%3dK %ls", c, (char far*)fent.name, len_kb, (char far*)fent.info);
		text_fill_char(x, y, LCD_CHAR_CX - x - 1, ' ');
	}
}

void draw_list_folder(Folder* folder, Rect *r_clip)
{
	int i;
	for(i = folder->home; i < folder->home + LIST_COUNT && i < folder->files; i++) {
		draw_list1(folder, i);
	}
	set_color(CLR_DESKTOP);
	i -= folder->home - FOLDER_Y;
	for(; i < STATUS_Y; i++) {
		text_fill_char(0, i, LCD_CHAR_CX, ' ');
	}
}

void draw_folder(Folder* folder, Rect *r_clip)
{
	if(folder->flags & FOLDER_LIST)
		draw_list_folder(folder, r_clip);
	else
		draw_icon_folder(folder, r_clip);
}

WORD do_folder(Folder* folder)
{
	int select0;
	BOOL scroll;
	WORD key;
	for(;;) {
		draw_scroll_arrow(folder);
		select0 = folder->select;
		scroll = FALSE;
		key = remote.mode ? key_hit_check_with_repeat() : key_wait();
		if(folder->flags & FOLDER_LIST) {
			switch(key) {
			case KEY_DOWN1:
				if(select0 < folder->files - 1) {
					if(++folder->select >= folder->home + LIST_COUNT) {
						folder->home++;
						scroll = TRUE;
					}
				}
				break;
			case KEY_UP1:
				if(select0) {
					if(--folder->select < folder->home) {
						folder->home --;
						scroll = TRUE;
					}
				}
				break;
			default:
				return key;
			}
		} else {
			switch(key) {
			case KEY_RIGHT1:
				folder->select++;
				goto ScrollDown;
			case KEY_LEFT1:
				if(select0 > 0) folder->select--;
				goto ScrollUp;
			case KEY_DOWN1:
				if((select0 / ICON_COUNT_X + 1) * ICON_COUNT_X < folder->files) {
					folder->select += ICON_COUNT_X;
	ScrollDown:		if(folder->select >= folder->files) folder->select = folder->files - 1;
					if(folder->select >= folder->home + (ICON_COUNT_X * ICON_COUNT_Y)) {
						folder->home += ICON_COUNT_X;
						scroll = TRUE;
					}
				}
				break;
			case KEY_UP1:
				if(select0 - ICON_COUNT_X >= 0)	 folder->select -= ICON_COUNT_X;
	ScrollUp:	if(folder->select < folder->home) {
					folder->home -= ICON_COUNT_X;
					scroll = TRUE;
				}
				break;
			default:
				return key;
			}
		}
		if(folder->files) {
			if(scroll)
				draw_folder(folder, NULL);
			else {
				if(folder->flags & FOLDER_LIST) {
					draw_list1(folder, select0);
					draw_list1(folder, folder->select);
				} else {
					draw_icon_on_desk(folder, select0);
					draw_icon_on_desk(folder, folder->select);
				}
			}
		}
	}
}

void draw_status(Folder* folder)
{
	int x;
	int space = _lrotr(get_free_space(), 10);
	set_color(CLR_STATUS);
	x = text_printf(0, STATUS_Y, "%s: %d files, %dKB free", fs_name[folder->fsid], folder->files, space);
	text_fill_char(x, STATUS_Y, LCD_CHAR_CX - x, ' ');
}
