/*
 *	User Interface base implementation
 *	issued: 2000/4/2
 *	author: c.mos@zob.ne.jp
 */

#include "gui.h"

#define BLANK_LINE_Y (LCD_CHAR_CY - 2)
#define FONT_BLANK 0

#ifndef screen_get_char1
WORD screen_get_char1(int scr, int x, int y);
#endif

static int font_sprite;
int sprite_used = 0;

int colors[CLR_COUNT] = { 3, 3, 11, 2, 8, 3, 11 };

void set_color(int i)
{
	font_set_color(colors[i]);
}

void reserve_sprite_font()
{
	int font = screen_get_char1(SCREEN2, 0, BLANK_LINE_Y) & CFM_FONT;
	if(font) {
		font_sprite = font;
		screen_fill_char(SCREEN2, 0, BLANK_LINE_Y, LCD_CHAR_CX, 1, PAL_DESKTOP << PALSFT | FONT_BLANK);
		font_set_far_colordata(font_sprite,	 SPRITE_FONT_COUNT, get_bmpres(BMP_FRAME));
	}
}

static void set_frame_sprite(int x, int y, int n, int font)
{
	sprite_set_char_location(n, (PAL_FRAME << PALSFT | CFM_SPR_UPPER) | font + font_sprite, x * 8, y * 8);
}

void draw_frame(Rect *r)
{
	int i;
	int n = sprite_used;

	set_frame_sprite(r->x - 1, r->y - 1, n++, FRAME_TOP_L);
	for_to(i, r->cx)
		set_frame_sprite(r->x + i, r->y - 1, n++, FRAME_TOP);
	set_frame_sprite(r->x + r->cx, r->y - 1, n++, FRAME_TOP_R);
	for_to(i, r->cy) {
		set_frame_sprite(r->x - 1, r->y + i, n++, FRAME_L);
		set_frame_sprite(r->x + r->cx, r->y + i, n++, FRAME_R);
	}
	set_frame_sprite(r->x - 1, r->y + r->cy, n++, FRAME_BTM_L);
	for_to(i, r->cx)
		set_frame_sprite(r->x + i, r->y + r->cy, n++, FRAME_BTM);
	set_frame_sprite(r->x + r->cx, r->y + r->cy, n++, FRAME_BTM_R);

	sprite_set_range(0, n);
	sprite_used = n;
}

void erase_frame(Rect *r)
{
	sprite_used -= (r->cx + r->cy) * 2 + 4;
	sprite_set_range(0, sprite_used);
}

void set_palette_rect(Rect *r, int pal)
{
	screen_fill_attr(SCREEN2, r->x, r->y, r->cx, r->cy, pal << PALSFT, ~CFM_PALETTE);
}

void fill_rect(Rect *r, int pal, int col)
{
	int y;
	set_palette_rect(r, pal);
	set_color(col);
	for_to(y, r->cy) {
		text_fill_char(r->x, y + r->y, r->cx, ' ');
	}
}

#ifndef screen_get_char1
WORD screen_get_char1(int scr, int x, int y)
{
	static int code;
	screen_get_char(scr, x, y, 1, 1, &code);
	return code;
}
#endif

void put_color_font(int x, int y, WORD far *font)
{
	WORD code = screen_get_char1(SCREEN2, x, y) & CFM_FONT;
	font_set_far_colordata(code, 1, font);
}

void put_color_icon(int x, int y, int cx, int cy, WORD far *icon)
{
	int y1;
	for_to(y1, cy) {
		WORD code = screen_get_char1(SCREEN2, x, y + y1) & CFM_FONT;
		font_set_far_colordata(code, cx, icon);
		icon += cx * 8;
	}
}

/*
void put_mono_icon(int x, int y, int cx, int cy, BYTE *icon)
{
	int y1;
	for_to(y1, cy) {
		WORD code = screen_get_char1(SCREEN2, x, y + y1) & CFM_FONT;
		font_set_monodata(code, cx, icon);
		icon += cx * 8;
	}
}
*/

int string_split(char far * far *pp, int splitter)
{
	char far *p = *pp;
	char c;
	int width = 0;
	if(p) {
		while((c = *p++)) {
			if((BYTE)c <= splitter) break;
			if(ismblead(c)) p++;
			width++;
		}
		*pp = p-1;
	}
	return width;
}

int string_size(char far *p, Size far *size)
{
	size->cx = size->cy = 0;
	for(;;) {
		int width = string_split(&p, '\n');
		size->cy++;
		if(width > size->cx) size->cx = width;
		if(*p++ == '\0') break;
	}
	return size->cx;
}

int string_width(char far *p, int len)
{
	char c;
	int width = 0;
	int n = 0;
	if(p) {
		while((c = *p++)) {
			if(len && n >= len) break;
			n++;
			if(ismblead(c)) { p++; n++; }
			width++;
		}
	}
	return width;
}

int string_skip(char far *p, int width_max)
{
	char c;
	int width = 0;
	int n = 0;
	if(p) {
		while((c = *p++)) {
			if(width >= width_max) break;
			n++;
			if(ismblead(c)) { p++; n++; }
			width++;
		}
	}
	return n;
}

/* rectangular management */

BOOL intersect_rect(Rect *r, const Rect *r1, const Rect *r2)
{
	Rect rdummy;
	if(!r) r = &rdummy;
	r->x  = max(r1->x, r2->x);
	r->cx = min(r1->x+r1->cx, r2->x+r2->cx) - r->x;
	r->y  = max(r1->y, r2->y);
	r->cy = min(r1->y+r1->cy, r2->y+r2->cy) - r->y;
	return r->cx > 0 && r->cy > 0;
}

void inflate_rect(Rect *r, int dx, int dy)
{
	r->x -= dx;
	r->y -= dy;
	r->cx += dx*2;
	r->cy += dy*2;
}

#if 0

BOOL pointin_rect(const Rect *r, int x, int y)
{
	return x >= r->x && x < r->x + r->cx && y >= r->y && y < r->y + r->cy;
}

BOOL Rect::Clip(Point& ptDst, Rect& rClip)
{
	Rect r(ptDst, GetSize());
	if(!r.Intersect(r, rClip)) return FALSE;
	Size d(GetPoint() - ptDst);
	ptDst = r.GetPoint();
	r += d;
	Set(r);
	return TRUE;
}

#endif

void mask_font(int x, int y, WORD far *fontdata)
{
	int i;
	WORD far *fontp;
	WORD *buffp;
	static int code;
	static WORD fontbuff[8];

	code = screen_get_char1(SCREEN2, x, y) & CFM_FONT;
	font_get_data(code, 1, fontbuff);
	fontp = fontdata;
	buffp = fontbuff;
	for_to(i, 8) {
		WORD mask = (*((BYTE far*)fontp) | *(((BYTE far*)fontp)+1));
		mask |= mask<<8;
		*buffp = *buffp & ~mask | *fontp & mask;
		buffp++; fontp++;
	}
	font_set_colordata(code, 1, fontbuff);
}

void draw_button_frame(Rect *r)
{
	int i;
	WORD far *bmp_frame = get_bmpres(BMP_FRAME);
	mask_font(r->x - 1, r->y - 1, bmp_frame + (FRAME_TOP_L * 8));
	for_to(i, r->cx)
		mask_font(r->x + i, r->y - 1, bmp_frame + (FRAME_TOP * 8));
	mask_font(r->x + r->cx, r->y - 1, bmp_frame + (FRAME_TOP_R * 8));
	for_to(i, r->cy) {
		mask_font(r->x - 1, r->y + i, bmp_frame + (FRAME_L * 8));
		mask_font(r->x + r->cx, r->y + i, bmp_frame + (FRAME_R * 8));
	}
	mask_font(r->x - 1, r->y + r->cy, bmp_frame + (FRAME_BTM_L * 8));
	for_to(i, r->cx)
		mask_font(r->x + i, r->y + r->cy, bmp_frame + (FRAME_BTM * 8));
	mask_font(r->x + r->cx, r->y + r->cy, bmp_frame + (FRAME_BTM_R * 8));
}

void draw_separator(int x, int y, int width, int bmp, int pal)
{
	int i;
	screen_fill_attr(SCREEN2, x, y, width, 1, pal << PALSFT, ~CFM_PALETTE);
	for_to(i, width)
		mask_font(x + i, y, (WORD far*)get_bmpres(BMP_FRAME) + (bmp * 8));
}

/*** bmp resource access ***/

extern res_t far *_resource;

void far *get_bmpres(int n)
{
	res_t far *res = _resource;
	while(n--) {
		if(res->magic != RESID) return NULL;
		(BYTE far*)res += (res->size << 4);
	}
	return (BYTE far*)res + sizeof(res_image_t);
}


void font_set_far_colordata(unsigned char_no, unsigned char_count, void far *data_p)
{
	int _ds_ = _DS;
	_DS = FP_SEG(data_p);
	_DX = FP_OFF(data_p);
	_CX = char_count;
	_BX = char_no;
	_AH = FONT_SET_COLORDATA;
	__int__(INT_DISP);
	_DS = _ds_;
}
