/*
 *	remote protocol manager implementation
 *	issued: 2000/7/23
 *	author: c.mos@zob.ne.jp
 */

#include "ged.h"
#include "remote.h"

enum { REMOTE_STATE_NONE, REMOTE_STATE_WAIT, REMOTE_STATE_BUSY, REMOTE_STATE_READY };

enum { CMD_LS, CMD_PUT, CMD_EXEC, CMD_DEFLAG, CMD_COUNT };

static char *cmd_string[CMD_COUNT] = { "ls", "put", "exec", "defrag" };

static char CODE_125[] = "125 STARTING";
static char CODE_200[] = "200 OK";
static char CODE_350[] = "350 FURTHER INFO";

Remote remote;

void far remote_callback()
{
	int c;
	c = comm_receive_char();
	if(c < 0) {
		remote.state = c;
		remote.count = 0;
	} else if(c == '\r') {
		remote.buffer[remote.count] = '\0';
		remote.state = REMOTE_STATE_READY;
	} else if(c >= ' ' && remote.count < REMOTE_BUFFER_SIZE - 1) {
		remote.buffer[remote.count++] = c;
		remote.state = REMOTE_STATE_BUSY;
	}
}

void remote_enable()
{
	static intvector_t vector;
	remote.state = REMOTE_STATE_WAIT;
	remote.count = 0;
	vector.callback = (void (near *)())remote_callback;
	vector.cs = _CS;
	vector.ds = _DS;
	sys_interrupt_set_hook(SYS_INT_RECEIVEREADY, &vector, NULL);
	comm_open();
	lcd_set_segments(LCDSEG_AUX1);
}

void remote_disable()
{
	comm_close();
	sys_interrupt_reset_hook(SYS_INT_RECEIVEREADY, NULL);
	remote.state = REMOTE_STATE_NONE;
	lcd_set_segments(0);
}

void init_remote_mode(BOOL mode)
{
	lcd_set_segments(0);
	if(mode != remote.mode) {
		if(mode)
			remote_enable();
		else
			remote_disable();
		remote.mode = mode;
	}
}

void send_line(char *s)
{
	comm_send_string(s);
	comm_send_string("\r\n");
}

int get_fmode_string(char *buff, fmode_t mode)
{
	return sprintf(buff, "%c%c%c%c%c",
		mode & FMODE_DIR	 ? 'd' :
		mode & FMODE_LINK	 ? 'l' :
		mode & FMODE_ILIB	 ? 'i' : '-',
		mode & FMODE_R		 ? 'r' : '-',
		mode & FMODE_W		 ? 'w' : '-',
		mode & FMODE_X		 ? 'x' : '-',
		mode & FMODE_STREAM ? '=' : '-'
	);
}

int get_mtime_string(char *buff, time_t mtime, BOOL date)
{
	datetime_t dt;
	time_t2datetime(mtime, &dt);
	if(date)
		return sprintf(buff, "%d-%02d-%02d ", dt.year + 2000, dt.month, dt.date);
	else
		return sprintf(buff, "%02d:%02d:%02d ", dt.hour, dt.minute, dt.second);
}



void set_fs(Folder *folder)
{
	char *p = (char near*)strchr(remote.buffer, '/');
	FSID fsid = (p && *(p+2)=='a') ? FS_SRAM : FS_FLASH;
	if(folder->fsid != fsid) {
		folder->fsid = fsid;
		init_folder(folder, TRUE);
		draw_desktop(NULL);
	}
}

void do_ls(void)
{
	int i;
	int files;
	files = get_file_count();
	sprintf(msgbuff, "total %d files", files);
	send_line(msgbuff);
	for_to(i, files) {
		fent_t fent;
		char *p;
		get_entry_tmp(i, &fent);
		p = msgbuff + lstrncpy(msgbuff, fent.name, MAXFNAME);
		while(p < msgbuff + 17) *p++ = ' ';
		p += get_fmode_string(p, fent.mode);
		p += sprintf(p, "%7lu ", fent.len);
		p += get_mtime_string(p, fent.mtime, TRUE);
		p += get_mtime_string(p, fent.mtime, FALSE);
	    p += lstrncpy(p, fent.info, MAXFINFO);
	    *p = '\0';
	    send_line(msgbuff);
	}
	send_line(".");
}

int do_remote(Folder *folder)
{
	int i;
	if(remote.state == REMOTE_STATE_READY) {
		remote.state = REMOTE_STATE_WAIT;
		remote.count = 0;
		for_to(i, CMD_COUNT) {
			char *s = cmd_string[i];
			if(!strncmp(remote.buffer, s, strlen(s))) {
				switch(i) {
				case CMD_LS:
					send_line(CODE_125);
					set_fs(folder);
					set_mask(0);
					do_ls();
					set_mask(folder->flags & FOLDER_SYS ? 0 : FMODE_ILIB);
					goto Done;
				case CMD_PUT:
					set_fs(folder);
					send_line(CODE_350);
					return ID_FILE_LOAD;
				case CMD_EXEC:
					send_line(CODE_200);
					return ID_FILE_EXEC;
				case CMD_DEFLAG:
					send_line(CODE_125);
					return ID_TOOL_DEFLAG;
				}
			}
		}
		send_line(CODE_125);
Done:	send_line(CODE_200);
	}
	return 0;
}

void done_remote(BOOL ok)
{
	if(remote.mode) {
		if(ok)
			send_line(CODE_200);
	}
}
